import requests

from Pythactyl import Admin

HEADERS = {
    "Accept": "application/json",
    "Content-type": "application/json"
}

class User:
    def __init__(self, resp: dict):
        self.email = resp["email"]
        self.username = resp["username"]
        self.fname = resp["first_name"]
        self.lname = resp["last_name"]
        self.lang = resp["language"]
        self.admin = resp["root_admin"]
        self.id = resp["uuid"]
        self.twofactor = resp["2fa"]
        self.type = "User"
        self.created_at = resp["created_at"]

    def __repr__(self):
        return f"<Object User>"

class Node:
    def __init__(self, resp):
        self.id = resp['id']
        self.uuid = resp['uuid']
        self.public = resp['public']
        self.name = resp['name']
        self.description = resp['description']
        self.location_id = resp['location_id']
        self.fqdn = resp['fqdn']
        self.scheme = resp['scheme']
        self.behind_proxy = resp['behind_proxy']
        self.maintenance_mode = resp['maintenance_mode']
        self.memory = resp['memory']
        self.memory_overallocate = resp['memory_overallocate']
        self.disk = resp['disk']
        self.disk_overallocate = resp['disk_overallocate']
        self.upload_size = resp['upload_size']
        self.daemon_listen = resp['daemon_listen']
        self.daemon_sftp = resp['daemon_sftp']
        self.daemon_base = resp['daemon_base']
        self.created_at = resp['created_at']
        self.updated_at = resp['updated_at']
        self.allocated_resources = resp['allocated_resources']
    def __repr__(self):
        return "<Object Node>"

class NodeConfig:
    def __init__(self, resp: dict):
        self.debug = resp['debug']
        self.uuid = resp['uuid']
        self.token_id = resp['token_id']
        self.token = resp['token']
        self.api_host = resp['api']['host']
        self.api_port = resp['api']['port']
        self.api_sll = resp['api']['ssl']['enabled']
        self.sll_cert = resp['api']['ssl']['cert']
        self.ssl_key = resp['api']['ssl']['key']
        self.api_upload_limit = resp['api']['upload_limit']
        self.system = resp['system']['data']
        self.sftp = resp['system']['sftp']['bind_port']
        self.allowed_mounts = resp['allowed_mounts']
        self.remote_uri = resp['remote']

class Allocation:
    def __init__(self, resp: dict):
        self.id = resp['id']
        self.ip = resp['ip']
        self.alias = resp['alias']
        self.port = resp['port']
        self.notes = resp['notes']
        self.assigned = resp['assigned']

class Key:
    def __init__(self, identifier=None, description=None, allowed_ips=None, lastused=None, created_at=None, token=None):
        self.id = identifier
        self.description = description
        self.allowed_ips = allowed_ips
        self.lastusedat = lastused
        self.created_at = created_at
        self.token = token

    def __repr__(self):
        return f"<{self.token if self.token else self.id}>"

class Location:
    def __init__(self, resp: dict):
        self.id = resp['id']
        self.short = resp['short']
        self.long = resp['long']
        self.created_at = resp['created_at']
        self.updated_at = resp['updated_at']


class Server:
    def __init__(self, resp):
        self.id = resp['id']
        self.identifier = resp['identifier']
        self.uuid = resp['uuid']
        self.name = resp['name']
        self.node = resp['node']
        self.description = resp['description']
        self.memory = resp['limits']['memory']
        self.swap = resp['limits']['swap']
        self.disk_space = resp['limits']['disk']
        self.io = resp['limits']['io']
        self.cpu = resp['limits']['cpu']
        self.thread = resp['limits']['threads']
        self.oom_disabled = resp['limits']['oom_disabled']

        self.databases = resp['feature_limits']['databases']
        self.allocation = resp['feature_limits']['allocations']
        self.backups = resp['feature_limits']['backups']
        self.owner_id = resp['user']
        self.node_id = resp['node']
        self.allocation_id = resp['allocation']
        self.egg_id = resp['egg']

        self.startup_command = resp['container']['startup_command']
        self.image = resp['container']['image']
        self.installed = resp['container']['installed']
        self.enviroment = resp['container']['environment']
        self.created_at = resp['created_at']
        self.updated_at = resp['updated_at']
        self.suspended = resp['suspended']


class SFTP:
    def __init__(self, ip=None, port=None):
        self.ip = ip
        self.por = port

class ServerLimits:
    def __init__(self, memory=None, swap=None, disk=None, io=None, cpu=None):
        self.memory = memory
        self.swap = swap
        self.disk = disk
        self.io = io
        self.cpu = cpu

class FeatureLimits:
    def __init__(self, databases=None, allocations=None, backups=None):
        self.databases = databases
        self.allocations = allocations
        self.backups = backups

class Database:
    def __init__(self, resp):
        self.id = resp['id']
        self.server_id = resp['server']
        self.node_id = resp['host']
        self.database = resp['database']
        self.db_user = resp['username']
        self.remote = resp['remote']
        self.max_connections = resp['max_connections']
        self.root_pass = resp['relationships']['password']['attributes']['password']
        self.host_id = resp['host']['attributes']['id']
        self.host_port =resp['host']['attributes']['port']
        self.host_username = resp['host']['attributes']['username']
        self.host_node_id = resp['host']['attributes']['node']
        self.created_at = resp['host']['attributes']['created_at']
        self.updated_at = resp['host']['attributes']['updated_at']

class Nest:
    def __init__(self, resp):
        self.id = resp['id']
        self.uuid = resp['uuid']
        self.author = resp['author']
        self.name = resp['name']
        self.description = resp['description']
        self.created_at = resp['created_at']
        self.updated_at = resp['updated_at']

class Egg:
    def __init__(self, resp):
        self.id = resp['id']
        self.uuid = resp['uuid']
        self.name = resp['name']
        self.nest = resp['nest']
        self.author = resp['author']
        self.description = resp['description']
        self.docker_image = resp['docker_image']
        self.stop_command = resp['config']['stop']
        self.startup_commmad = resp['startup']
        self.priveleged = resp['script']['privileged']
        self.install_script = resp['script']['install']
        self.container = resp['script']['container']


class Relationship:
    def __init__(self, id=None, ip=None, ip_alias=None, port=None, notes=None, default=None):
        self.id = id
        self.ip = ip
        self.ip_alias = ip_alias
        self.port = port
        self.notes = notes
        self.default = default


import requests
from .objects import User, Node, NodeConfig, Allocation, Location, Server, HEADERS, Database, Nest, Egg
from .Errors import UserExists
class PterodactylAdmin(object):
    def __init__(self, url, api_key):
        self.url = url + "/api/application"
        self.api_key = api_key
        self.headers = HEADERS
        self.headers['Authorization'] = f"Bearer {self.api_key}"

    def listUsers(self):
        r = requests.get(self.url + "/users", headers=self.headers)
        users = []
        for item in r.json()['data']: # Get list of users
            users.append(User(item['attributes']))

        return users

    def getUser(self, userid: int):
        r = requests.get(self.url + "/users/" + str(userid), headers=self.headers)
        data = r.json()['attributes']
        user = User(data)
        return user

    def getByExternalID(self, exid):
        r = requests.get(self.url + "/users/external/" + str(exid), headers=self.headers)
        return r.json()

    def createUser(self, email, username, first, last, language="en", root_admin=False):
        data = {
            "email": email,
            "username": username,
            "first_name": first,
            "last_name": last,
            "language": language,
            "root_admin": root_admin
        }
        r = requests.post(self.url + "/users", headers=self.headers, json=data)

        if r.status_code == 422:
            raise UserExists("User already exists")

        data = r.json()['attributes']

        user = User(data)
        return user

    def editUser(self, userid, email, username, first_name, lastname, language, password, root_admin=None):
        data = {}
        data['email'] = email
        data['username'] = username
        data['first_name'] = first_name
        data['last_name'] = lastname
        data['language'] = language
        data['password'] = password
        data['root_admin'] = root_admin if root_admin else None

        r = requests.patch(self.url + "/users/" + str(userid), headers=self.headers, json=data)
        return User(r.json()['attributes'])

    def removeUser(self, userid):
        r = requests.delete(self.url + "/users/" + str(userid), headers=self.headers)
        return r.status_code == 204

    #   Nodes

    def listNodes(self):
        r = requests.get(self.url + "/nodes", headers=self.headers)
        nodes = []
        for data in r.json()['data']:
            node = Node(data['attributes'])
            nodes.append(node)
        return nodes

    def getNode(self, nodeid):
        r = requests.get(self.url + "/nodes/" + str(nodeid), headers=self.headers)
        return Node(r.json()['attributes'])

    def getNodeConfig(self, nodeid):
        r = requests.get(self.url + "/nodes/" + str(nodeid) + "/configuration", headers=self.headers)
        return NodeConfig(r.json())

    def createNode(self, name, location, fqdn, memory, disk,  scheme="https", memory_over=0, disk_over=0, upload_size=100, daemon_sftp=2022, daemon_listen=8080):
        data = {
            "name": name,
            "location_id": location,
            "fqdn": fqdn,
            "scheme": scheme,
            "memory": memory,
            "memory_overallocate": memory_over,
            "disk": disk,
            "disk_overallocate": disk_over,
            "upload_size": upload_size,
            "daemon_sftp": daemon_sftp,
            "daemon_listen": daemon_listen
        }
        r = requests.post(self.url + "/nodes", headers=self.headers, json=data)
        return Node(r.json()['attributes'])

    def updateNode(self, name=None, location=None, fqdn=None, scheme=None, memory=None, memory_over=None, disk=None, disk_over=None, upload_size=None, daemon_sftp=None, daemon_listen=None):
        data = {}
        if name:
            data['name'] = name
        if location:
            data['location_id'] = location
        if fqdn:
            data['fqdn'] = fqdn
        if scheme:
            data['scheme'] = scheme
        if memory:
            data['memory'] = memory
        if memory_over:
            data['memory_overallocate'] = memory_over
        if disk:
            data['disk'] = disk
        if upload_size:
            data['upload_size'] = upload_size
        if daemon_sftp:
            data['daemon_sftp'] = daemon_sftp
        if daemon_listen:
            data['daemon_listen'] = daemon_listen

        r = requests.put(self.url + "/nodes", headers=self.headers, json=data)
        return Node(r.json()['attributes'])

    def removeNode(self, nodeid):
        r = requests.delete(self.url + "/nodes/" + str(nodeid), headers=self.headers)
        return r.status_code == 204

    #   Allocations

    def listAllocations(self, nodeid):
        r = requests.get(self.url + "/nodes/" + str(nodeid) + "/allocations", headers=self.headers)
        allocations = []
        for allocation in r.json()['data']:
            allocations.append(Allocation(allocation['attributes']))

        return allocations

    def createAllocation(self, nodeid, ip, ports=[]):
        if len(ports) <= 0:
            raise SyntaxError("You must specify atleast 1 port")
        data = {
            "ip": ip,
            "ports": [str(a) for a in ports]
        }
        r = requests.post(self.url + "/nodes/" + str(nodeid) + "/allocations", headers=self.headers, json=data)
        return r.status_code == 204

    def removeAllocation(self, nodeid, allocationid):
        r = requests.delete(self.url + "/nodes/" + str(nodeid) + "/allocations/" + str(allocationid), headers=self.headers)
        return r.status_code == 204

    #   Locations

    def listLocations(self):
        r = requests.get(self.url + "/locations", headers=self.headers)
        locs = []
        for loc in r.json()['data']:
            locs.append(Location(loc['attributes']))
        return locs

    def getLocation(self, locationid):
        r = requests.get(self.url + "/locations/" + str(locationid), headers=self.headers)
        return Location(r.json()['attributes'])

    def createLocation(self, short, long):
        if len(short) > 4:
            raise TypeError("Your short location name must be less than 4 characters long")
        data = {
            "short": short,
            "long": long
        }
        r = requests.post(self.url + "/locations", headers=self.headers, json=data)
        return Location(r.json()['attributes'])

    def updateLocation(self, locationid, short=None, long=None):
        data = {}
        if short:
            if len(short) > 4:
                raise TypeError("Your short location name must be less than 4 characters long")
            data['short'] = short
        if long:
            data['long'] = long
        r = requests.put(self.url + "/locations/" + str(locationid), headers=self.headers, json=data)
        return Location(r.json()['attributes'])

    def removeLocation(self, locationid):
        r = requests.delete(self.url + "/locations/" + str(locationid), headers=self.headers)
        return r.status_code == 204

    #   Servers

    def listServers(self):
        r = requests.get(self.url + "/servers", headers=self.headers)
        _servers = []
        for serv in r.json()['data']:
            _servers.append(Server(serv['attributes']))
        return _servers

    def getServer(self, serverid):
        r = requests.get(self.url + "/servers/" + str(serverid), headers=self.headers)
        # setattr(self, "name", r[''])
        return Server(r.json()['attributes'])

    def getServerByExternalID(self, extid):
        r = requests.get(self.url + "/servers/external/" + str(extid), headers=self.headers)
        return r.json()

    def updateServer(self, serverid, name=None, user=None, externalid=None, description=None):
        data = {}
        if name:
            data['name'] = name
        if user:
            data['user'] = user
        if externalid:
            data['external_id'] = externalid
        if description:
            data['description'] = description

        r = requests.put(self.url + "/servers/" + str(serverid) + "/details", headers=self.headers, json=data)
        return Server(r.json()['attributes'])

    def updateServerBuild(self, serverid, allocation, memory=None, swap=None, io=None, cpu=None, disk=None, threads=None, feature_limits={'databases': 0, 'allocations': 3,'backups': 1}):
        data = {
            "allocation": allocation
        }
        if memory:
            data['memory'] = memory
        if swap:
            data['swap'] = swap
        if io:
            data['io'] = io
        if cpu:
            data['cpu'] = cpu
        if threads:
            data['threads'] = threads
        if feature_limits:
            data['feature_limits'] = feature_limits
        r = requests.put(self.url + "/servers/" + str(serverid) + "/build", headers=self.headers, json=data)
        return Server(r.json()['attributes'])

    def updateServerStartup(self, serverid, startup, egg, image, environment: dict=None, skip_scripts=False):
        data = {
            "startup": startup,
            "egg": egg,
            "image": image,
            "skip_scripts": skip_scripts
        }
        if environment:
            data['environment'] = environment

        r = requests.put(self.url + "/servers/" + str(serverid) + "/startup", headers=self.headers, json=data)
        return Server(r.json()['attributes'])

    def createServer(self, name, userid, eggid, image, startup, enviroment: dict, limits: dict, feature_limits: dict, allocation: dict):
        data = {
            'name': name,
            "user": userid,
            "egg": eggid,
            "docker_image": image,
            "startup": startup,
            "enviroment": enviroment,
            "limits": limits,
            "feature_limits": feature_limits,
            "allocation": allocation
        }
        r = requests.post(self.url + "/servers", headers=self.headers, json=data)
        print(r.content)
        return Server(r.json()['attributes'])

    def suspendServer(self, serverid):
        r = requests.post(self.url + "/servers/" + str(serverid) + "/suspend", headers=self.headers)
        return r.status_code == 204

    def unsuspendServer(self, serverid):
        r = requests.post(self.url + "/servers/" + str(serverid) + "/unsuspend", headers=self.headers)
        return r.status_code == 204

    def reinstallServer(self, serverid):
        r = requests.post(self.url + "/servers/" + str(serverid) + "/reinstall", headers=self.headers)
        return r.status_code == 204

    def deleteServer(self, serverid):
        r = requests.delete(self.url + "/servers/" + str(serverid), headers=self.headers)
        return r.status_code == 204

    def forceDeleteServer(self, serverid):
        r = requests.delete(self.url + "/servers/" + str(serverid) + "/force", headers=self.headers)
        return r.status_code == 204

    #   Server Databases

    def listServerDatabases(self, serverid, include_pass_host=False):
        r = requests.get(self.url + "/servers/" + str(serverid) + f"/databases{'?include=password,host' if include_pass_host else ''}", headers=self.headers)
        databases = []
        for db in r.json()['data']:
            databases.append(Database(db['attributes']))
        return databases

    def getServerDBDetails(self, serverid, db_id, include_pass_host=False):
        r = requests.get(self.url + "/servers/" + str(serverid) + f"/databases/" + str(db_id) + f"{'?include=password,host' if include_pass_host else ''}", headers=self.headers, json=data)
        return Database(r.json()['attributes'])

    def createServerDatabase(self, serverid ,db, remote="%", host=None):
        data = {
            "database": db,
            "remote": remote,
        }
        if host:
            data['host'] = host
        r = requests.post(self.url + "/servers/" + str(serverid) + "/databases", headers=self.headers, json=data)
        return Database(r.json()['attributes'])

    def resetServerDBPassword(self, serverid, dbid):
        r = requests.post(self.url + "/servers/" + str(serverid) + "/databases/" + str(db_id) + "/reset-password", headers=self.headers)
        return r.status_code == 204

    def removeServerDB(self, serverid, dbid):
        r = requests.delete(self.url + "/servers/" + str(serverid) + "/databases/" + str(db_id), headers=self.headers)
        return r.status_code == 204

    #   Nests

    def listNests(self, include_params=False):
        r = requests.get(self.url + f"/nests{'?include=eggs,servers' if include_params else ''}", headers=self.headers)
        nests = []
        for nest in r.json()['data']:
            nests.append(Nest(nest['attributes']))
        return nests

    def getNest(self, nestid, include_params=False):
        r = requests.get(self.url + f"/nests/{nestid}{'?include=eggs,servers' if include_params else ''}", headers=self.headers)
        return Nest(r.json()['attributes'])

    def listNestEggs(self, nestid, include_params=False):
        r = requests.get(self.url + f"/nests/{nestid}/eggs{'?include=nest,servers' if include_params else ''}", headers=self.headers)
        print(r.json())
        eggs = []
        for egg in r.json()['data']:
            eggs.append(Egg(egg['attributes']))
        return eggs

    def getNestEgg(self, nestid, eggid, include_params=False):
        r = requests.get(self.url + f"/nests/{nestid}/eggs/{eggid}{'?include=nest,servers,config,script,variables' if include_params else ''}", headers=self.headers)
        return Egg(r.json()['attributes'])


if __name__ == "__main__":
    print("I dont run. No use in trying to make me lol")



import requests
from .objects import User, HEADERS

class PterodactylClient(object):
    def __init__(self, url, api_key):
        self.url = url + "/api/client"
        self.api_key = api_key
        self.headers = HEADERS
        self.headers['Authorization'] = f"Bearer {self.api_key}"

    def account(self):
        r = requests.get(self.url + "/account", headers=self.headers).json()
        r = r['attributes']
        return User(
            id=r['id'],
            email=r['email'],
            admin=r['admin'],
            fname=r['first_name'],
            lname=r['lname'],
            lang=r['language'],
            twofactor=self._check2fa()['data']  # This returns a QR code image
        )

    def _check2fa(self):
        r = requests.get(self.url + "/account/two-factor", headers=self.headers)
        return r.json()

    def updateEmail(self, email, password):
        data = {
            "email": email,
            "password": password
        }
        r = requests.put(self.url + "/account/email", headers=self.headers, json=data)
        return r.status_code == 201

    def updatePassword(self, password, newpass, confpass):
        data = {
            "current_password": password,
            "password": newpass,
            "password_confirmation": confpass
        }
        r = requests.put(self.url + "/account/email", headers=self.headers, json=data)
        return r.status_code == 204

    def listApikeys(self):
        r = requests.get(self.url + "/account/api-keys", headers=self.headers).json()
        return [].append([x for x in r['data']])

    def createApiKey(self, description, allowed_ips=[]):
        data = {
            "description": description,
            "allowed_ips": allowed_ips
        }
        r = requests.post(self.url + "/account/api-keys", headers=self.headers, json=data).json()
        meta = r['meta']
        r = r['attributes']
        return objects.Key(
            identifer=r['identifier'],
            description=r['description'],
            allowed_ips=r['allowed_ips'],
            lastused=r['last_used_at'],
            created_at=r['created_at'],
            token=meta['secret_token']
        )

    def removeApiKey(self, code):
        r = requests.delete(self.url + "/account/api-keys/" + str(code), headers=self.headers)
        if r.status_code == 404:
            return {'error': 'An error has occured','tips': 'Check the code is correct and/or existing'}
        else:
            return r.status_code == 204
    #       Servers
    def listServers(self):
        r = requests.get(self.url, headers=self.headers).json()
        _servers = []
        for server in r['data']:
            _relationships = []
            for x in server['relationships']['allocations']['data']:
                _relationships.append(
                    objects.Relationship(x['attributes']['id'], x['attributes']['ip'], x['attributes']['ip_alias'],
                                         x['attributes']['port'], x['attributes'], x['attributes']['notes'],
                                         x['attributes']['is_default']))
            _servers.append(objects.Server(
                owner=server['owner'],
                identifier=server['identifier'],
                uuid=server['uuid'],
                name=server['name'],
                node=server['node'],
                sftp=objects.SFTP(server['sftp_details']['ip'], server['sftp_details']['port']),
                description=server['description'],
                limits=objects.ServerLimits(server['limits']['memory'], server['limits']['swap'], server['limits']['disk'], server['limits']['io'], server['limits']['cpu']),
                feature_limits=objects.FeatureLimits(server['feature_limits']['databases'], server['feature_limits']['allocations'], server['feature_limits']['backups']),
                suspended=server['is_suspended'],
                installing=server['is_installing'],
                relationships=_relationships
            ))
        return _servers

    def getServer(self, identifier):
        r = requests.get(self.url + "/servers/" + str(identifier), headers=self.headers)
        return r.json()

    def sendPowerAction(self, identifier, action):
        signals = ['start', 'stop','restart','kill']
        if action.lower() not in signals:
            return {'error': 'Incorrect signal sent','available signals': signal}
        data = {
            "signal": action
        }
        r = requests.post(self.url + "/servers/" + identifier + "/power", headers=self.headers, json=data)
        return r

    def sendCommand(self, identifier, command):
        data = {
            "command": command
        }
        r = requests.post(self.url + "/servers/" + identifier + "/command", headers=self.headers, json=data)
        return r

    def listDatabases(self, identifier):
        r = requests.get(self.url + "/servers/" + str(identifier) + "/databases", headers=self.headers)
        return r.json()

    def createDatabase(self, identifier, db_name, remote_addr="%"):
        data = {
            "database": db_name,
            "remote": remote_addr
        }
        r = requests.post(self.url + "/servers/" + str(identifier) + "/databases", headers=self.headers, json=data)
        return r.json()

    def resetDatabasePassword(self, identifier, db_id):
        r = requests.post(self.url + "/servers/" + str(identifier) + "/databases/" + db_id + "/rotate-password", headers=self.headers)
        return r.json()

    def removeDatabase(self, identifier, db_id):
        r = requests.delete(self.url + "/servers/" + str(identifier) + "/databases/" + db_id + "rotate-password", headers=self.headers)
        return r.json()

    #   Subusers

    def listSubusers(self, identifier):
        r = requests.get(self.url + "/servers/" + str(identifier) + "/users", headers=self.headers)
        return r.json()

    def addSubuser(self, identifier, email, permissions=[]):
        if len(permissions) <= 0:
            raise PermissionsMissing("You must specify at least 1 permission node")
        data = {
            "email": email,
            "permissions": permissions
        }
        r = requests.post(self.url + "/servers/" + str(identifier) + "/users", headers=self.headers, json=data)
        return r.json()

    def getSubuser(self, identifier, uuid):
        r = requests.get(self.url + "/servers/" + str(identifier) + "/users/" + uuid, headers=self.headers)
        return r.json()

    def updateSubuser(self, identifier, uuid, permissions=[]):
        if len(permissions) <= 0:
            raise PermissionsMissing("You must specify at least 1 permission node")
        data = {
            "permissions": permissions
        }
        r = requests.post(self.url + "/servers/" + str(identifier) + "/users/" + uuid, headers=self.headers, json=data)
        return r.json()

    def removeSubuser(self, identifier, uuid):
        r = requests.delete(self.url + "/servers/" + str(identifier) + "/users/" + uuid, headers=self.headers, json=data)
        return r.json()

if __name__ == "__main__":
    print("I dont run. No use in trying to make me lol")
